@@ -61,4 +61,4 @@ class MeanDCShift(Analyzer):
         dc_result = self.new_result(data_mode='value', time_mode='global')
         dc_result.data_object.value = numpy.round(
             numpy.mean(100 * self.values), 3)
-        self.pipe.results.add(dc_result)
+        self.process_pipe.results.add(dc_result)
