@@ -22,16 +22,17 @@
 from timeside.core import implements, interfacedoc
 from timeside.analyzer.core import Analyzer
 from timeside.api import IAnalyzer
-from utils import downsample_blocking
 import numpy as np
 
+from preprocessors import downmix_to_mono, frames_adapter
 
 class Waveform(Analyzer):
     implements(IAnalyzer)  # TODO check if needed with inheritance
 
     def __init__(self):
-        self.input_blocksize = 2048
-        self.input_stepsize = self.input_blocksize / 2
+        super(Waveform, self).__init__()
+#        self.input_blocksize = 2048
+#        self.input_stepsize = self.input_blocksize / 2
 
     @interfacedoc
     def setup(self, channels=None, samplerate=None,
@@ -57,12 +58,13 @@ class Waveform(Analyzer):
     def unit():
         return ""
 
+#    @downmix_to_mono
+#    @frames_adapter
     def process(self, frames, eod=False):
-        for samples in downsample_blocking(frames, self.input_blocksize):
-            self.values.append(samples)
+        self.values.append(frames)
         return frames, eod
 
     def post_process(self):
         waveform = self.new_result(data_mode='value', time_mode='framewise')
-        waveform.data_object.value = np.asarray(self.values).flatten()
-        self.pipe.results.add(waveform)
+        waveform.data_object.value = np.vstack(self.values)
+        self.process_pipe.results.add(waveform)
