@@ -112,6 +112,12 @@ class Yaafe(Analyzer):
             result.id_metadata.name += ' ' + featName
             # Read Yaafe Results
             result.data_object.value = self.yaafe_engine.readOutput(featName)
+
+            yaafe_metadata = self.yaafe_engine.getOutputs()[featName]
+            result.frame_metadata.blocksize = yaafe_metadata['frameLength']
+            result.frame_metadata.stepsize = yaafe_metadata['sampleStep']
+            result.frame_metadata.samplerate = yaafe_metadata['sampleRate']
+
             # Store results in Container
             if len(result.data_object.value):
-                self.pipe.results.add(result)
+                self.process_pipe.results.add(result)
